summaryrefslogtreecommitdiff
path: root/gcc/tree-ssa-phiopt.c
diff options
context:
space:
mode:
authorrguenth <rguenth@138bc75d-0d04-0410-961f-82ee72b054a4>2010-10-20 11:09:54 +0000
committerrguenth <rguenth@138bc75d-0d04-0410-961f-82ee72b054a4>2010-10-20 11:09:54 +0000
commit3211fa0aac7a557f2fa98b17e52b25896b30b523 (patch)
tree787b901537ce872da945a332f8791b8d5726a19e /gcc/tree-ssa-phiopt.c
parent85cea2e300bb24c3736eb946ae0af0fb711705f3 (diff)
downloadgcc-3211fa0aac7a557f2fa98b17e52b25896b30b523.tar.gz
2010-10-20 Richard Guenther <rguenther@suse.de>
PR tree-optimization/45860 * tree-ssa-phiopt.c (cond_store_replacement): Do not do conditional store replacement for non-register type stores. * gcc.dg/torture/pr45860.c: New testcase. git-svn-id: svn+ssh://gcc.gnu.org/svn/gcc/trunk@165718 138bc75d-0d04-0410-961f-82ee72b054a4
Diffstat (limited to 'gcc/tree-ssa-phiopt.c')
-rw-r--r--gcc/tree-ssa-phiopt.c9
1 files changed, 4 insertions, 5 deletions
diff --git a/gcc/tree-ssa-phiopt.c b/gcc/tree-ssa-phiopt.c
index 482f267a648..8555bc1ef6f 100644
--- a/gcc/tree-ssa-phiopt.c
+++ b/gcc/tree-ssa-phiopt.c
@@ -1204,10 +1204,11 @@ cond_store_replacement (basic_block middle_bb, basic_block join_bb,
|| TREE_CODE (TREE_OPERAND (lhs, 0)) != SSA_NAME)
return false;
/* RHS is either a single SSA_NAME or a constant. */
code = gimple_assign_rhs_code (assign);
if (get_gimple_rhs_class (code) != GIMPLE_SINGLE_RHS
- || (code != SSA_NAME && !is_gimple_min_invariant (rhs)))
+ || (code != SSA_NAME && !is_gimple_min_invariant (rhs))
+ || !is_gimple_reg_type (TREE_TYPE (lhs)))
return false;
/* Prove that we can move the store down. We could also check
TREE_THIS_NOTRAP here, but in that case we also could move stores,
@@ -1217,8 +1218,8 @@ cond_store_replacement (basic_block middle_bb, basic_block join_bb,
/* Now we've checked the constraints, so do the transformation:
1) Remove the single store. */
- mark_symbols_for_renaming (assign);
gsi = gsi_for_stmt (assign);
+ unlink_stmt_vdef (assign);
gsi_remove (&gsi, true);
/* 2) Create a temporary where we can store the old content
@@ -1237,7 +1238,6 @@ cond_store_replacement (basic_block middle_bb, basic_block join_bb,
name = make_ssa_name (condstoretemp, new_stmt);
gimple_assign_set_lhs (new_stmt, name);
gimple_set_location (new_stmt, locus);
- mark_symbols_for_renaming (new_stmt);
gsi_insert_on_edge (e1, new_stmt);
/* 4) Create a PHI node at the join block, with one argument
@@ -1249,7 +1249,6 @@ cond_store_replacement (basic_block middle_bb, basic_block join_bb,
lhs = unshare_expr (lhs);
new_stmt = gimple_build_assign (lhs, PHI_RESULT (newphi));
- mark_symbols_for_renaming (new_stmt);
/* 5) Insert that PHI node. */
gsi = gsi_after_labels (join_bb);
@@ -1204,10 +1204,11 @@ cond_store_replacement (basic_block middle_bb, basic_block join_bb,
|| TREE_CODE (TREE_OPERAND (lhs, 0)) != SSA_NAME)
return false;
- /* RHS is either a single SSA_NAME or a constant. */
+ /* RHS is either a single SSA_NAME or a constant of register type. */
code = gimple_assign_rhs_code (assign);
if (get_gimple_rhs_class (code) != GIMPLE_SINGLE_RHS